@@ -1,19 +1,18 @@
-import sys
 import unittest
+import sys
 
 from flaky import flaky
 import pytest
+
 from qtpy import QtCore, QtGui, QtWidgets
 from qtpy.QtTest import QTest
 
 from qtconsole.console_widget import ConsoleWidget
 from qtconsole.qtconsoleapp import JupyterQtConsoleApp
+
 from . import no_display
 
-if sys.version[0] == '2':  # Python 2
-    from IPython.core.inputsplitter import InputSplitter as TransformerManager
-else:
-    from IPython.core.inputtransformer2 import TransformerManager
+from IPython.core.inputtransformer2 import TransformerManager
 
 
 SHELL_TIMEOUT = 20000
@@ -26,10 +25,12 @@ def qtconsole(qtbot):
     console = JupyterQtConsoleApp()
     console.initialize(argv=[])
 
-    qtbot.addWidget(console.window)
     console.window.confirm_exit = False
     console.window.show()
-    return console
+
+    yield console
+
+    console.window.close()
 
 
 @flaky(max_runs=3)
@@ -155,10 +156,7 @@ def test_input(qtconsole, qtbot):
     with qtbot.waitSignal(shell.executed):
         shell.execute("import time")
 
-    if sys.version[0] == '2':
-        input_function = 'raw_input'
-    else:
-        input_function = 'input'
+    input_function = 'input'
     shell.execute("print(" + input_function + "('name: ')); time.sleep(3)")
 
     qtbot.waitUntil(lambda: control.toPlainText().split()[-1] == 'name:')
@@ -242,9 +240,9 @@ class TestConsoleWidget(unittest.TestCase):
                        'foo\b\nbar\n',
                        'foo\b\nbar\r\n',
                        'abc\rxyz\b\b=']
-        expected_outputs = [u'x=z\u2029',
-                            u'foo\u2029bar\u2029',
-                            u'foo\u2029bar\u2029',
+        expected_outputs = ['x=z\u2029',
+                            'foo\u2029bar\u2029',
+                            'foo\u2029bar\u2029',
                             'x=z']
         for i, text in enumerate(test_inputs):
             w._insert_plain_text(cursor, text)
@@ -252,11 +250,35 @@ class TestConsoleWidget(unittest.TestCase):
             # clear all the text
             cursor.insertText('')
 
+    def test_erase_in_line(self):
+        """ Do control sequences for clearing the line work?
+        """
+        w = ConsoleWidget()
+        cursor = w._get_prompt_cursor()
+
+        test_inputs = ['Hello\x1b[1KBye',
+                       'Hello\x1b[0KBye',
+                       'Hello\r\x1b[0KBye',
+                       'Hello\r\x1b[1KBye',
+                       'Hello\r\x1b[2KBye',
+                       'Hello\x1b[2K\rBye']
+
+        expected_outputs = ['     Bye',
+                            'HelloBye',
+                            'Bye',
+                            'Byelo',
+                            'Bye',
+                            'Bye']
+        for i, text in enumerate(test_inputs):
+            w._insert_plain_text(cursor, text)
+            self.assert_text_equal(cursor, expected_outputs[i])
+            # clear all the text
+            cursor.insertText('')
+
     def test_link_handling(self):
-        noKeys = QtCore.Qt
-        noButton = QtCore.Qt.MouseButton(0)
-        noButtons = QtCore.Qt.MouseButtons(0)
-        noModifiers = QtCore.Qt.KeyboardModifiers(0)
+        noButton = QtCore.Qt.NoButton
+        noButtons = QtCore.Qt.NoButton
+        noModifiers = QtCore.Qt.NoModifier
         MouseMove = QtCore.QEvent.MouseMove
         QMouseEvent = QtGui.QMouseEvent
 
@@ -265,23 +287,23 @@ class TestConsoleWidget(unittest.TestCase):
         w._insert_html(cursor, '<a href="http://python.org">written in</a>')
         obj = w._control
         tip = QtWidgets.QToolTip
-        self.assertEqual(tip.text(), u'')
+        self.assertEqual(tip.text(), '')
 
         # should be somewhere else
-        elsewhereEvent = QMouseEvent(MouseMove, QtCore.QPoint(50,50),
+        elsewhereEvent = QMouseEvent(MouseMove, QtCore.QPointF(50, 50),
                                      noButton, noButtons, noModifiers)
         w.eventFilter(obj, elsewhereEvent)
         self.assertEqual(tip.isVisible(), False)
-        self.assertEqual(tip.text(), u'')
+        self.assertEqual(tip.text(), '')
         # should be over text
-        overTextEvent = QMouseEvent(MouseMove, QtCore.QPoint(1,5),
+        overTextEvent = QMouseEvent(MouseMove, QtCore.QPointF(1, 5),
                                     noButton, noButtons, noModifiers)
         w.eventFilter(obj, overTextEvent)
         self.assertEqual(tip.isVisible(), True)
         self.assertEqual(tip.text(), "http://python.org")
 
         # should still be over text
-        stillOverTextEvent = QMouseEvent(MouseMove, QtCore.QPoint(1,5),
+        stillOverTextEvent = QMouseEvent(MouseMove, QtCore.QPointF(1, 5),
                                          noButton, noButtons, noModifiers)
         w.eventFilter(obj, stillOverTextEvent)
         self.assertEqual(tip.isVisible(), True)
@@ -361,6 +383,7 @@ class TestConsoleWidget(unittest.TestCase):
         copied = app.clipboard().text()
         self.assertEqual(copied,  'Header\nprompt>if:\n>     pass')
 
+    @pytest.mark.skipif(sys.platform == 'darwin', reason="Fails on macOS")
     def test_keypresses(self):
         """Test the event handling code for keypresses."""
         w = ConsoleWidget()
@@ -549,7 +572,7 @@ class TestConsoleWidget(unittest.TestCase):
         w._handle_is_complete_reply(
             dict(parent_header=dict(msg_id=msg_id),
                  content=dict(status="incomplete", indent="!!!")))
-        self.assert_text_equal(cursor, u"thing\u2029> !!!")
+        self.assert_text_equal(cursor, "thing\u2029> !!!")
         self.assertEqual(calls, [])
 
         # test complete statement (_execute called)
@@ -562,16 +585,16 @@ class TestConsoleWidget(unittest.TestCase):
                  content=dict(status="complete", indent="###")))
         self.assertEqual(calls, [("else", False)])
         calls = []
-        self.assert_text_equal(cursor, u"thing\u2029> !!!else\u2029")
+        self.assert_text_equal(cursor, "thing\u2029> !!!else\u2029")
 
         # test missing answer from is_complete
         msg_id = object()
         w.execute("done", interactive=True)
         self.assertEqual(calls, ["done"])
         calls = []
-        self.assert_text_equal(cursor, u"thing\u2029> !!!else\u2029")
+        self.assert_text_equal(cursor, "thing\u2029> !!!else\u2029")
         w._trigger_is_complete_callback()
-        self.assert_text_equal(cursor, u"thing\u2029> !!!else\u2029\u2029> ")
+        self.assert_text_equal(cursor, "thing\u2029> !!!else\u2029\u2029> ")
 
         # assert that late answer isn't destroying anything
         w._handle_is_complete_reply(
